/*
 * Created by $USER$ on $DATE$
 * All Rights Reserved
 */

#ifndef _$FILENAME_CAPS$_
#define _$FILENAME_CAPS$_

#import <Foundation/NSView.h>

@interface $FILENAME_EXT$ : NSView
{
  // Instance variables
}

// Class methods



// Instance methods



@end

#endif // _$FILENAME_CAPS$_
