/*
    NSSetAdditions.m

    Implementations of the additions to the NSSet class for the
    ProjectManager application.

    Copyright (C) 2005  Saso Kiselkov

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import "NSSetAdditions.h"

#import "NSDictionaryAdditions.h"
#import "NSArrayAdditions.h"

@implementation NSSet (PMAdditions)

/**
 * Same as -[NSArray makeDeeplyMutableEquivalent].
 *
 * @see [NSArray makeDeeplyMutableEquivalent]
 */
- makeDeeplyMutableEquivalent
{
  NSMutableSet * mutableCopy;
  NSEnumerator * e;
  id object;
  Class arrayClass = [NSArray class],
        dictionaryClass = [NSDictionary class],
        setClass = [NSSet class];

  mutableCopy = [NSMutableSet setWithCapacity: [self count]];
  e = [self objectEnumerator];
  while ((object = [e nextObject]) != nil)
    {
      if ([object isKindOfClass: arrayClass] ||
          [object isKindOfClass: dictionaryClass] ||
          [object isKindOfClass: setClass])
        {
          [mutableCopy addObject: [object makeDeeplyMutableEquivalent]];
        }
      else
        {
          [mutableCopy addObject: object];
        }
    }

  return mutableCopy;
}

@end
