/*
    EditorFontButton.m

    Implementation of the EditorFontButton class for the
    ProjectManager application.

    Copyright (C) 2005  Saso Kiselkov

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import "EditorFontButton.h"

#import <AppKit/NSTextField.h>
#import <AppKit/NSFontManager.h>

#import "EditorLooksPreferences.h"

@implementation EditorFontButton

- (void) doFontChange: sender
{
  [[NSFontManager sharedFontManager]
    setSelectedFont: [textField font] isMultiple: NO];
  [[NSFontManager sharedFontManager] orderFrontFontPanel: nil];
}

- (void) changeFont: sender
{
  NSFont * newFont;

  newFont = [sender convertFont: [textField font]];
  [textField setFont: newFont];
  [textField setStringValue: [NSString stringWithFormat:
    @"%@ %.1fpt", [newFont fontName], [newFont pointSize]]];

  [owner fontChangedInField: textField];
}

@end
