/* All Rights Reserved */

#import "Document.h"
#import <AppKit/AppKit.h>

@implementation Document

- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) aType
{
  /* Insert code here to read-in your document from `data'. */

  return NO;
}

- (NSData *) dataRepresentationOfType: (NSString *) aType
{
  /* Insert code here to return a data representation of your document. */
  
  return nil;
}

- (NSString *) windowNibName
{
  return @"Document";
}

@end
