/*
    ApplicationTypeAttribubtes.h

    Interface declaration of the ApplicationTypeAttributes class for the
    ProjectManager application.

    Copyright (C) 2005, 2006  Saso Kiselkov

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import <Foundation/NSObject.h>

@class NSMutableArray, ProjectDocument;
@class NSTableView, NSTableColumn, NSView;

@interface ApplicationTypeAttributes : NSObject
{
  NSMutableArray * typeInfo;

  // weak reference
  ProjectDocument * document;

  id bogusWindow;
  id description;
  id documentClass;
  id icon;
  id iconFilename;
  id extensions;
  id table;
  id view;
}

- initWithTypesArray: (NSMutableArray *) types
            document: (ProjectDocument *) doc;

- (void) add: (id)sender;
- (void) changeDescription: (id)sender;
- (void) changeDocumentClass: (id)sender;
- (void) changeIconFilename: (id)sender;
- (void) changeExtensions: (id)sender;
- (void) remove: (id)sender;

- (void) showType: sender;

- (NSView *) view;

- (int) numberOfRowsInTableView: (NSTableView *)aTableView;
- (id) tableView: (NSTableView *)aTableView 
objectValueForTableColumn: (NSTableColumn *)aTableColumn 
             row: (int)rowIndex;
- (void) tableView: (NSTableView *)aTableView 
    setObjectValue: (id)anObject 
    forTableColumn: (NSTableColumn *)aTableColumn
               row: (int)rowIndex;

@end
