/*
    RelativePathUtilities.h

    Interfaces of extensions to the NSString class to provide
    relative path handling.

    Authors: Matt Rice  <ratmice@yahoo.com>
             Saso Kiselkov  <diablos@manga.sk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import <Foundation/NSString.h>

@interface NSString (RelativePathUtilities)

- (NSString *) stringByConstructingRelativePathTo: (NSString *) otherPath;

- (NSString *) stringByConcatenatingWithPath: (NSString *) otherPath;

@end
