/*
    ProjectModule.m

    Implementation of the ProjectModule class for the ProjectManager
    application.

    Copyright (C) 2005, 2006  Saso Kiselkov

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import "ProjectModule.h"

#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSDictionary.h>

/**
 * Posted when the current project module changes. The userInfo
 * dictionary contains a single key:
 *
 * Module = <module>;
 *
 * which is currently the active one, or if no current project module
 * is active (or has been deactivated), the key will be absent.
 */
NSString * const CurrentProjectModuleDidChangeNotification =
  @"CurrentProjectModuleDidChangeNotification";
