/* All Rights Reserved */

#ifndef _DOCUMENT_H_
#define _DOCUMENT_H_

#import <AppKit/AppKit.h>

@interface Document : NSDocument

- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) aType;
- (NSData *) dataRepresentationOfType: (NSString *) aType;

- (NSString *) windowNibName;

@end

#endif // _DOCUMENT_H_
